﻿using ProjectAccordionBus.Exceptions;
using ProjectAirbus.Exceptions;
using ProjectAccordionBus.Drawnings;

namespace ProjectAccordionBus.CollectionGenericObjects;

/// <summary>
/// Параметризированный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов которые храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;


    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException(position);

        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        if (comparer != null)
        {
            foreach (T? item in _collection)
            {
                if ((comparer as IEqualityComparer<DrawningBus>).Equals(obj as DrawningBus, item as DrawningBus))
                {
                    throw new ObjectNotUniqueException();
                }
            }
        }
        // TODO вставка в свободное место набора
        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        else
        {
            for (int i = position + 1; i < Count; i++)
            {
                if (_collection[i] == null)
                {
                    _collection[i] = obj;
                    return i;
                }
            }

            for (int i = 0; i < position; i++)
            {
                if (_collection[i] == null)
                {
                    _collection[i] = obj;
                    return i;
                }
            }
        }

        throw new CollectionOverflowException(Count);
    }

    public T? Remove(int position)
    {
        if (position >= Count || position < 0)
        {
            throw new PositionOutOfCollectionException(position);
        }

        T? obj = _collection[position];
        if (obj == null)
        {
            throw new ObjectNotFoundException(position);
        }
        _collection[position] = null;
        return obj;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }

    void ICollectionGenericObjects<T>.CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}

