﻿using Microsoft.Extensions.Logging;
using ProjectAccordionBus.CollectionGenericObjects;
using ProjectAccordionBus.Drawnings;
using ProjectAccordionBus.Exceptions;
using ProjectAirbus.Exceptions;

namespace ProjectAccordionBus;

/// <summary>
/// 
/// </summary>
public partial class FormBusCollection : Form
{
    /// <summary>
    /// Хранилише коллекций
    /// </summary>
    private readonly StorageCollection<DrawningBus> _storageCollection;

    /// <summary>
    /// 
    /// </summary>
    private AbstractCompany? _company = null;

    private readonly ILogger _logger;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBusCollection(ILogger<FormBusCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
    }
    /// <summary>
    /// Добавление улучшенного троллейбуса
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddAccordionBus_Click(object sender, EventArgs e)
    {
        FormBusConfig form = new FormBusConfig();
        form.Show();
        form.AddEvent(SetBus);
    }

    private void buttonAddBus_Click(object sender, EventArgs e)
    {
        FormBusConfig form = new();
        form.Show();
        form.AddEvent(SetBus);
    }

    private void SetBus(DrawningBus bus)
    {
        try
        {
            if (_company == null || bus == null)
            {
                return;
            }

            if (_company + bus != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Добавлен объект: " + bus.GetDataForSave());
            }
        }
        catch (ObjectNotFoundException ex)
        {
        }
        catch (CollectionOverflowException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
        catch (ObjectNotUniqueException ex)
        {
            MessageBox.Show("Такой объект уже присутствует в коллекции");
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    private void buttonDeleteBus_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);
        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Удален объект по позиции " + pos);
            }
            else
            {
                MessageBox.Show($"Не удалось удалить объект");
            }
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningBus? bus = null;
        int counter = 100;
        while (bus == null)
        {
            try
            {
                bus = _company.GetRandomObject();
            }
            catch (ObjectNotFoundException)
            {
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
        }
        if (bus == null)
        {
            return;
        }
        FormAccordionBus form = new()
        {
            SetBus = bus
        };
        form.ShowDialog();
    }

    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    private void buttonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        try
        {
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RefreshListBoxItems();
            _logger.LogInformation("Коллекция добавлена " + textBoxCollectionName.Text);
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    private void RefreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i].Name;
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }


    private void buttonCollectionDel_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);
        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Удален объект по позиции " + pos);
            }
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    private void buttonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            _logger.LogWarning("Создание компании невыбранной коллекции");
            return;
        }
        ICollectionGenericObjects<DrawningBus>? collection =
        _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            _logger.LogWarning("Не удалось инициализировать коллекцию");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new BusSharingService(pictureBox.Width,
                pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RefreshListBoxItems();
    }

    private void saveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void loadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Загрузка из файла: {filename}", saveFileDialog.FileName);
                RefreshListBoxItems();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void CompareBus(IComparer<DrawningBus?> comparer)
    {
        if (_company == null)
        {
            return;
        }

        _company.Sort(comparer);
        pictureBox.Image = _company.Show();
    }

    private void buttonSortByType_Click(object sender, EventArgs e)
    {
        CompareBus(new DrawningBusCompareByType());
    }

    private void buttonSortByColor_Click(object sender, EventArgs e)
    {
        CompareBus(new DrawningBusCompareByColor());
    }
}
